/* Test the tester.  */
/* { dg-prms-id 42 } */
/* { dg-options "-Wall" } */
/* { dg-do compile } */
/* { dg-excess-errors "excess errors" } */

/* No messages, so unexpected pass.  */

int main () { return 0; }
